<?php

/**
 * Attempts to determine our clients true IP address, looking for HTTP_CLIENT_IP
 * first, and checks if a user is behind a proxy.
 * 
 * @since 0.4.7
 * @ver 1.0
 * @return string Returns the best IP address found
 */

function kstats_get_real_ip() {

	$ip = FALSE;

	if ( ! empty( $_SERVER["HTTP_CLIENT_IP"] ) )
		$ip = $_SERVER["HTTP_CLIENT_IP"];

	if ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {

       // Put the IP's into an array which we shall work with shortly.
       $ips = explode( ', ', $_SERVER['HTTP_X_FORWARDED_FOR'] );

		if ( $ip ) {
			array_unshift( $ips, $ip );
			$ip = FALSE;
		}

		$num_ips = count( $ips );

		for ( $i = 0; $i < $num_ips; $i++ ) {

			// Skip RFC 1918 IP's 10.0.0.0/8, 172.16.0.0/12 and 192.168.0.0/16
			if ( ! preg_match( '~^(10|172\.16|192\.168)\.~', $ips[$i] ) ) {

				if ( ip2long( $ips[$i] ) != FALSE || ip2long( $ips[$i] ) != -1 ) {
                       $ip = $ips[$i];
                       break;
				}

			} // if

		} // for

	}

	return ( $ip ? $ip : $_SERVER['REMOTE_ADDR'] );

}


/**
 * @todo Use the new IP validation created for the recent pageviews controller here, then use this function there.
 */
 
function kstats_validate_ip( $ip, $fullip = FALSE, $fake = FALSE ) {


	// While the regex would catch extra groups, let's just accommodate
	// the possibility of typo's...
	$parts = explode( '.', $ip );

	if ( count( $parts ) > 4 )
		$parts = array_splice( $parts, 0, 4 );

	$ip = implode( '.', $parts );


	// This is (slightly) unadultered return value, upon validation
	$ret = $ip;


	// If $complete was TRUE, we'd be looking for an entire IP address. If it
	// is false, we'll just fill in the blanks for the regex validation.
	if ( $fullip === FALSE ) {

		$parts = explode( '.', $ip );

		$ip = array();

		for ( $i = 0; $i < 4; $i++ ) {
			$ip[] = ( isset( $parts[$i] ) && ! empty( $parts[$i] ) ? $parts[$i] : '0' );
		}

		$ip = implode( '.', $ip );

	}

	// Yes? No?
	$valid = ( preg_match( '~^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$~', $ip ) ? TRUE : FALSE );

	if ( $valid === TRUE )
		return $ret;
	else if ( $valid === FALSE && $fake === TRUE )
		return '127.0.0.1';

	return FALSE;

}


/**
 * Run the provided IP address against the list of ignored IPs
 *
 * @since 0.1.0
 * @param int $ip The IP address to be checked
 * @return bool True on match, false otherwise
 */

function kstats_ignore_ip( $ip  = '') {
	global $kstats;

	if ( empty( $ip ) || empty( $kstats->config['ignore']['ignore_list'] ) )
		return FALSE;

	foreach ( $kstats->config['ignore']['ignore_list'] as $ignore )
		if ( strpos( $ip, $ignore ) !== FALSE )
			return TRUE;

	return FALSE;

}


/**
 * Grab the requested URL and format it for use by the collector.
 *
 * @since 0.1.0
 * @return string The formatted request URI
 */

function kstats_get_url () {
	global $kstats;

	$url = esc_url( $_SERVER['REQUEST_URI'] );

	if ( substr ( $url, 0, 2) == '/?' )
		$url = substr ( $url, 1 );

	if ( $url == '/' )
		$url = $kstats->blog_url . '/';

	return $url;
}


/**
 * Attempts to determine if our visitor is a search engine spider or not, using
 * the INI file provided by user-agent-string.info
 *
 * @ver 0.5.0
 * @category Trap
 * @param string $user_agent The user agent string to run the query against
 * @return FALSE|string Returns the name of the spider if matched, false otherwise
 */

function kstats_trap_spider( $user_agent = '' ) {

	if ( empty( $user_agent ) || ! file_exists( KSTATS_INI_UA ) )
		return FALSE;

	$uas = parse_ini_file( KSTATS_INI_UA, TRUE );

	foreach ( $uas['robots'] as $test ) {
		if ( $test[0] == $user_agent )
			return $test[1];
	}

	return FALSE;
}


/**
 * Attempts to determine whether or not the visitor is accessing a feed, and which type
 *
 * @ver 0.1.0
 * @category Trap
 * @param string $url The url string to run the query against
 * @return FALSE|string Returns the type of feed if matched, false otherwise
 *
 * @todo Not very happy with the use of get_bloginfo for this, need to find a better method
 */

function kstats_trap_feed( $url = '' ) {

	if ( empty( $url ) || strpos( '+' . $url, 'feed' ) === FALSE )
		return FALSE;

	if ( stristr( $url, get_bloginfo( 'comments_atom_url' ) ) != FALSE )
		return 'COMMENT ATOM';
	else if ( stristr( $url, get_bloginfo( 'comments_rss2_url' ) ) != FALSE )
		return 'COMMENT RSS';
	else if ( stristr( $url, get_bloginfo( 'rdf_url' ) ) != FALSE )
		return 'RDF';
	else if ( stristr( $url, get_bloginfo( 'atom_url' ) ) != FALSE )
		return 'ATOM';
	else if ( stristr( $url, get_bloginfo( 'rss_url' ) ) != FALSE )
		return 'RSS';
	else if ( stristr( $url, get_bloginfo( 'rss2_url' ) ) != FALSE )
		return 'RSS2';
	elseif ( stristr( $url, 'wp-feed.php' ) != FALSE )
		return 'RSS2';
	else if ( stristr( $url, '/feed' ) != FALSE )
		return 'RSS2';

	return FALSE;
}


/**
 * Attempts to determine the visitors Operating System based on their User Agent
 *
 * @ver 0.1.0
 * @category Trap
 * @param string $user_agent The user agent string to run the query against
 * @return FALSE|string Returns the name of their OS if matched, false otherwise
 */

function kstats_trap_agent( $user_agent = '' ) {

	if ( empty( $user_agent ) || ! file_exists( KSTATS_INI_UA ) )
		return FALSE;

	$uas = parse_ini_file( KSTATS_INI_UA, TRUE );

	$browser = array();

	foreach ( $uas['browser_reg'] as $test ) {
		if ( preg_match( $test[0], $user_agent, $info ) ) { // $info[1] contains version if found
			$browser['id'] = $test[1];
			break;
		}
	}

	// Grab the Browser detail
	if ( isset( $browser['id'] ) ) {
		if ( $uas['browser_type'][$uas['browser'][$browser['id']][0]][0] ) {
			$browser['type'] = $uas['browser_type'][$uas['browser'][$browser['id']][0]][0];
			$browser['name'] = $uas['browser'][$browser['id']][1] . ' ' . $info[1]; // Attach the browsers version to it's name
		}
	}


	// Grab our OS
	foreach ( $uas['os_reg'] as $test ) {
		if ( preg_match( $test[0], $user_agent ) ) {
			$os = $test[1];
			break;
		}
	}

	if ( isset( $os ) )
		$browser['os'] = $uas['os'][$os][1];

	if ( ! empty( $browser ) )
		return $browser['os'] . '|' . $browser['name'];

	return FALSE;

}


/**
 * Attempts to determine any search terms used to find the site, based on the referrer
 *
 * @ver 0.1.0
 * @category Trap
 * @uses kstats_trap_search_terms
 * @param string $referrer The referrer string to run the query against
 * @return FALSE|string Returns the search terms if trapped, false otherwise
 */

function kstats_trap_search_query( $referrer ) {

	$data = parse_ini_file( KSTATS_INI_QUERY, TRUE );

	foreach ( $data as $url => $info ) {

		if ( strpos( $referrer, $url ) !== FALSE ) {

			$queries = kstats_trap_search_terms( $referrer );
			$i = count( $queries );

			while ( $i-- ) {
				$parts = explode( '=', $queries[$i] );
				if ( $parts[0] == $info['query'] )
					return ( $info['engine'] . '|' . urlencode ( $parts[1] ) );
			}

		}

	}
	
	return FALSE;
}


/**
 * Used by kstats_trap_search_query to pull the query string apart and perform a little kung-fu on it
 *
 * @ver 0.1.0
 * @category Trap
 * @param string $url The url string to run the query against
 * @return FALSE|array Returns an array of parts from the query string, false otherwise
 */

function kstats_trap_search_terms( $url ) {

	$query_str = parse_url( $url, PHP_URL_QUERY );

	if ( ! empty( $query_str ) )
		return explode( '&', html_entity_decode( urldecode( $query_str ) ) );

	return FALSE;
}


/**
 * Nothing special, simply returns the string with an ellipsis on the end
 * if it exceeds the requested length.
 *
 * @ver 0.1.0
 * @param string $string The string to truncate
 * @param int    $length The maximum length allowable in characters
 * @return string The original string, formatted if it exceeded $length, the same otherwise
 *
 * @todo Should this function return an ACRONYM automatically including the full string?
 */

function kstats_truncate( $string, $length ) {
	global $kstats;

	if ( $string == '/' || $string == $kstats->blog_url . '/' )
		return __( 'Page: Home', 'kstats' );
	if ( $length > strlen( $string ) )
		return $string;

	return mb_substr( $string, 0, $length ) . '...';
}


/**
 * Pulls data from the kStats totals table for use in building the overview table
 * and accompanying graphical bar chart.
 *
 * @ver 0.5.0
 * @category Database
 * @param int $days_to_total Should be grabbed from $kstats_options[settings_overview_chart|settings_dashboard_chart] depending on where the data is being displayed
 * @return array An array of data, including the charts days, totals, and $max for use in building the chart
 */

function kstats_get_totals () {
	global $wpdb, $kstats;

	// Set up the structure
	$arr = array( 'totals'  => array(),
				  'monthly' => array(),
				  'daily'   => array(),
				 );

	$arr['totals']['visitors'] = 0;
	$arr['totals']['pageviews'] = 0;
	$arr['totals']['spiders'] = 0;
	$arr['totals']['feeds'] = 0;

	$arr['daily'][$kstats->datetime->today]['visitors'] = 0;
	$arr['daily'][$kstats->datetime->today]['pageviews'] = 0;
	$arr['daily'][$kstats->datetime->today]['spiders'] = 0;
	$arr['daily'][$kstats->datetime->today]['feeds'] = 0;

	// Look for data in our aggregate table first
	$query[] = sprintf( "SELECT type, name, visitors, pageviews, spiders, feeds FROM `%s` ORDER BY name ", KSTATS_TABLE_TOTALS );
	$results = $wpdb->get_results( end( $query ), ARRAY_A );

	if ( ! is_null( $results ) ) {
		foreach ( $results as $data ) {
			if ( empty( $data['name'] ) )
				$arr['totals'] = array_slice( $data, 2 );
			else
				$arr[$data['type']][$data['name']] = array_slice( $data, 2 );
		}
	}


	// Then start pulling it from our raw table, if there is any...
	$query[] = sprintf( "SELECT SUBSTR( timestamp, 1, 10 ) AS timestamp, COUNT( DISTINCT ip ) AS visitors, COUNT( timestamp ) AS pageviews FROM `%s` WHERE spider = '' AND preserved IS NULL GROUP BY SUBSTR( timestamp, 1, 10 )", KSTATS_TABLE_RAW );
	$results = $wpdb->get_results( end( $query ), ARRAY_A );

	if ( ! is_null( $results ) ) {

		foreach ( $results as $row ) {
			if ( ! isset( $arr['daily'][$row['timestamp']]['visitors'] ) )
				$arr['daily'][$row['timestamp']]['visitors'] = 0;

			if ( ! isset( $arr['daily'][$row['timestamp']]['pageviews'] ) )
				$arr['daily'][$row['timestamp']]['pageviews'] = 0;

			$arr['daily'][$row['timestamp']]['visitors']  += $row['visitors'];
			$arr['daily'][$row['timestamp']]['pageviews'] += $row['pageviews'];


			$str_monthly = substr( $row['timestamp'], 0, 7 );

			if ( ! isset( $arr['monthly'][$str_monthly]['visitors'] ) )
				$arr['monthly'][$str_monthly]['visitors'] = 0;

			if ( ! isset( $arr['monthly'][$str_monthly]['pageviews'] ) )
				$arr['monthly'][$str_monthly]['pageviews'] = 0;

			$arr['monthly'][$str_monthly]['visitors']  += $row['visitors'];
			$arr['monthly'][$str_monthly]['pageviews'] += $row['pageviews'];


			$arr['totals']['visitors']  += $row['visitors'];
			$arr['totals']['pageviews'] += $row['pageviews'];
		}
	}


	$query[] = sprintf( "SELECT SUBSTR( timestamp, 1, 10 ) AS timestamp, COUNT( ip ) AS spiders FROM `%s` WHERE spider <> '' AND preserved IS NULL GROUP BY SUBSTR( timestamp, 1, 10 )", KSTATS_TABLE_RAW );
	$results = $wpdb->get_results( end( $query ), ARRAY_A );

	if ( ! is_null( $results ) ) {
		foreach ( $results as $row ) {
			if ( ! isset( $arr['daily'][$row['timestamp']]['spiders'] ) )
				$arr['daily'][$row['timestamp']]['spiders'] = 0;

			$arr['daily'][$row['timestamp']]['spiders'] += $row['spiders'];


			$str_monthly = substr( $row['timestamp'], 0, 7 );

			if ( ! isset( $arr['monthly'][$str_monthly]['spiders'] ) )
				$arr['monthly'][$str_monthly]['spiders'] = 0;

			$arr['monthly'][$str_monthly]['spiders'] += $row['spiders'];


			$arr['totals']['spiders'] += $row['spiders'];
		}
	}

	$query[] = sprintf( "SELECT SUBSTR( timestamp, 1, 10 ) AS timestamp, COUNT( ip ) AS feeds FROM `%s` WHERE feed <> '' AND preserved IS NULL GROUP BY SUBSTR( timestamp, 1, 10 )", KSTATS_TABLE_RAW );
	$results = $wpdb->get_results( end( $query ), ARRAY_A );

	if ( ! is_null( $results ) ) {
		foreach ( $results as $row ) {
			if ( ! isset( $arr['daily'][$row['timestamp']]['feeds'] ) )
				$arr['daily'][$row['timestamp']]['feeds'] = 0;

			$arr['daily'][$row['timestamp']]['feeds'] += $row['feeds'];


			$str_monthly = substr( $row['timestamp'], 0, 7 );

			if ( ! isset( $arr['monthly'][$str_monthly]['feeds'] ) )
				$arr['monthly'][$str_monthly]['feeds'] = 0;

			$arr['monthly'][$str_monthly]['feeds'] += $row['feeds'];


			$arr['totals']['feeds'] += $row['feeds'];
		}
	}

	ksort( $arr['monthly'] );
	ksort( $arr['daily'] );

	return $arr; // Should be extracted on the other side, but can be used literally too.

}


/**
 * Using the array provided by kstats_get_totals, this will return
 * a slice from that array for use by the bar graph, along with the
 * $max value necessary to build it.
 *
 * @since 0.6.0
 * @category Functions
 * @subcategory Display
 * @param int   $days  The number of days of data we want for the graph
 * @param array $daily The array of data generated by kstats_get_totals
 * @return array Associative array containing the array slice and max value
 */

function kstats_get_graph_data( $days = 21, $daily ) {
	global $kstats;

	if ( empty( $daily ) || ! is_array( $daily ) )
		return FALSE;

	$search = date( 'Y-m-d', strtotime( "-{$days} days", $kstats->datetime->timestamp ) );

	$slice = array_slice( $daily, array_search( $search, array_keys( $daily ) ) ); // ARRAY-FU!

	$max = array();

	foreach ( $slice as $data )
		$max[] = array_sum( $data );

	return array( $slice, max( $max ) );

}

function kstats_create_temp_key( $length = 6 ) {

	$charset = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';

	$key = '_';

	for ( $i = 0; $i < $length; $i++ )
		$key .= $charset[( mt_rand( 0, ( strlen( $charset ) - 1 ) ) )];

	return $key;

}
